/**
 * @file
 * VuoList_VuoArtNetOutputDevice C type definition.
 *
 * @copyright Copyright © 2012–2016 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoArtNetOutputDevice_H
#define VuoList_VuoArtNetOutputDevice_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoArtNetOutputDevice VuoList_VuoArtNetOutputDevice
 * A list of @ref VuoArtNetOutputDevice elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoArtNetOutputDevice_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoArtNetOutputDevice elements.
 */
typedef const struct VuoList_VuoArtNetOutputDevice_struct { void *l; } * VuoList_VuoArtNetOutputDevice;
/// @}
#endif

/**
 * Creates a new list of @ref VuoArtNetOutputDevice elements.
 */
VuoList_VuoArtNetOutputDevice VuoListCreate_VuoArtNetOutputDevice(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoArtNetOutputDevice to quickly initialize a large list.
 */
VuoList_VuoArtNetOutputDevice VuoListCreateWithCount_VuoArtNetOutputDevice(const unsigned long count, const VuoArtNetOutputDevice value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoArtNetOutputDevice VuoListCopy_VuoArtNetOutputDevice(const VuoList_VuoArtNetOutputDevice list);

/**
 * Returns the @ref VuoArtNetOutputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoArtNetOutputDevice VuoListGetValue_VuoArtNetOutputDevice(const VuoList_VuoArtNetOutputDevice list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoArtNetOutputDevice *VuoListGetData_VuoArtNetOutputDevice(const VuoList_VuoArtNetOutputDevice list);

/**
 * Changes the @ref VuoArtNetOutputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoArtNetOutputDevice(const VuoList_VuoArtNetOutputDevice list, const VuoArtNetOutputDevice value, const unsigned long index);

/**
 * Inserts the `VuoArtNetOutputDevice` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoArtNetOutputDevice(const VuoList_VuoArtNetOutputDevice list, const VuoArtNetOutputDevice value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoArtNetOutputDevice(VuoList_VuoArtNetOutputDevice list, const VuoArtNetOutputDevice value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoArtNetOutputDevice(VuoList_VuoArtNetOutputDevice list, const VuoArtNetOutputDevice value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoArtNetOutputDevice(VuoList_VuoArtNetOutputDevice list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoArtNetOutputDevice_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoArtNetOutputDevice(VuoList_VuoArtNetOutputDevice list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoArtNetOutputDevice_areEqual(const VuoList_VuoArtNetOutputDevice a, const VuoList_VuoArtNetOutputDevice b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoArtNetOutputDevice_isLessThan(const VuoList_VuoArtNetOutputDevice a, const VuoList_VuoArtNetOutputDevice b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoArtNetOutputDevice(VuoList_VuoArtNetOutputDevice list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoArtNetOutputDevice(VuoList_VuoArtNetOutputDevice list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoArtNetOutputDevice(VuoList_VuoArtNetOutputDevice list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoArtNetOutputDevice(VuoList_VuoArtNetOutputDevice list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoArtNetOutputDevice(VuoList_VuoArtNetOutputDevice list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoArtNetOutputDevice(VuoList_VuoArtNetOutputDevice list);

/**
 * Removes the `VuoArtNetOutputDevice` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoArtNetOutputDevice(VuoList_VuoArtNetOutputDevice list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoArtNetOutputDevice(const VuoList_VuoArtNetOutputDevice list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoArtNetOutputDevice VuoList_VuoArtNetOutputDevice_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoArtNetOutputDevice_getJson(const VuoList_VuoArtNetOutputDevice value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoArtNetOutputDevice_getSummary(const VuoList_VuoArtNetOutputDevice value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoArtNetOutputDevice VuoList_VuoArtNetOutputDevice_makeFromString(const char *str);
char * VuoList_VuoArtNetOutputDevice_getString(const VuoList_VuoArtNetOutputDevice value);
/// @}

/**
 * @}
 */

#endif
