/**
 * @file
 * VuoList_VuoDurationType C type definition.
 *
 * @copyright Copyright © 2012–2016 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoDurationType_H
#define VuoList_VuoDurationType_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoDurationType VuoList_VuoDurationType
 * A list of @ref VuoDurationType elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoDurationType_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoDurationType elements.
 */
typedef const struct VuoList_VuoDurationType_struct { void *l; } * VuoList_VuoDurationType;
/// @}
#endif

/**
 * Creates a new list of @ref VuoDurationType elements.
 */
VuoList_VuoDurationType VuoListCreate_VuoDurationType(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoDurationType to quickly initialize a large list.
 */
VuoList_VuoDurationType VuoListCreateWithCount_VuoDurationType(const unsigned long count, const VuoDurationType value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoDurationType VuoListCopy_VuoDurationType(const VuoList_VuoDurationType list);

/**
 * Returns the @ref VuoDurationType at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoDurationType VuoListGetValue_VuoDurationType(const VuoList_VuoDurationType list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoDurationType *VuoListGetData_VuoDurationType(const VuoList_VuoDurationType list);

/**
 * Changes the @ref VuoDurationType at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoDurationType(const VuoList_VuoDurationType list, const VuoDurationType value, const unsigned long index);

/**
 * Inserts the `VuoDurationType` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoDurationType(const VuoList_VuoDurationType list, const VuoDurationType value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoDurationType(VuoList_VuoDurationType list, const VuoDurationType value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoDurationType(VuoList_VuoDurationType list, const VuoDurationType value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoDurationType(VuoList_VuoDurationType list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoDurationType_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoDurationType(VuoList_VuoDurationType list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoDurationType_areEqual(const VuoList_VuoDurationType a, const VuoList_VuoDurationType b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoDurationType_isLessThan(const VuoList_VuoDurationType a, const VuoList_VuoDurationType b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoDurationType(VuoList_VuoDurationType list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoDurationType(VuoList_VuoDurationType list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoDurationType(VuoList_VuoDurationType list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoDurationType(VuoList_VuoDurationType list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoDurationType(VuoList_VuoDurationType list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoDurationType(VuoList_VuoDurationType list);

/**
 * Removes the `VuoDurationType` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoDurationType(VuoList_VuoDurationType list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoDurationType(const VuoList_VuoDurationType list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoDurationType VuoList_VuoDurationType_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoDurationType_getJson(const VuoList_VuoDurationType value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoDurationType_getSummary(const VuoList_VuoDurationType value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoDurationType VuoList_VuoDurationType_makeFromString(const char *str);
char * VuoList_VuoDurationType_getString(const VuoList_VuoDurationType value);
/// @}

/**
 * @}
 */

#endif
