/**
 * @file
 * VuoList_VuoLoopType C type definition.
 *
 * @copyright Copyright © 2012–2016 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoLoopType_H
#define VuoList_VuoLoopType_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoLoopType VuoList_VuoLoopType
 * A list of @ref VuoLoopType elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoLoopType_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoLoopType elements.
 */
typedef const struct VuoList_VuoLoopType_struct { void *l; } * VuoList_VuoLoopType;
/// @}
#endif

/**
 * Creates a new list of @ref VuoLoopType elements.
 */
VuoList_VuoLoopType VuoListCreate_VuoLoopType(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoLoopType to quickly initialize a large list.
 */
VuoList_VuoLoopType VuoListCreateWithCount_VuoLoopType(const unsigned long count, const VuoLoopType value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoLoopType VuoListCopy_VuoLoopType(const VuoList_VuoLoopType list);

/**
 * Returns the @ref VuoLoopType at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoLoopType VuoListGetValue_VuoLoopType(const VuoList_VuoLoopType list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoLoopType *VuoListGetData_VuoLoopType(const VuoList_VuoLoopType list);

/**
 * Changes the @ref VuoLoopType at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoLoopType(const VuoList_VuoLoopType list, const VuoLoopType value, const unsigned long index);

/**
 * Inserts the `VuoLoopType` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoLoopType(const VuoList_VuoLoopType list, const VuoLoopType value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoLoopType(VuoList_VuoLoopType list, const VuoLoopType value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoLoopType(VuoList_VuoLoopType list, const VuoLoopType value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoLoopType(VuoList_VuoLoopType list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoLoopType_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoLoopType(VuoList_VuoLoopType list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoLoopType_areEqual(const VuoList_VuoLoopType a, const VuoList_VuoLoopType b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoLoopType_isLessThan(const VuoList_VuoLoopType a, const VuoList_VuoLoopType b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoLoopType(VuoList_VuoLoopType list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoLoopType(VuoList_VuoLoopType list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoLoopType(VuoList_VuoLoopType list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoLoopType(VuoList_VuoLoopType list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoLoopType(VuoList_VuoLoopType list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoLoopType(VuoList_VuoLoopType list);

/**
 * Removes the `VuoLoopType` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoLoopType(VuoList_VuoLoopType list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoLoopType(const VuoList_VuoLoopType list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoLoopType VuoList_VuoLoopType_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoLoopType_getJson(const VuoList_VuoLoopType value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoLoopType_getSummary(const VuoList_VuoLoopType value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoLoopType VuoList_VuoLoopType_makeFromString(const char *str);
char * VuoList_VuoLoopType_getString(const VuoList_VuoLoopType value);
/// @}

/**
 * @}
 */

#endif
