/**
 * @file
 * VuoList_VuoOscOutputDevice C type definition.
 *
 * @copyright Copyright © 2012–2016 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoOscOutputDevice_H
#define VuoList_VuoOscOutputDevice_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoOscOutputDevice VuoList_VuoOscOutputDevice
 * A list of @ref VuoOscOutputDevice elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoOscOutputDevice_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoOscOutputDevice elements.
 */
typedef const struct VuoList_VuoOscOutputDevice_struct { void *l; } * VuoList_VuoOscOutputDevice;
/// @}
#endif

/**
 * Creates a new list of @ref VuoOscOutputDevice elements.
 */
VuoList_VuoOscOutputDevice VuoListCreate_VuoOscOutputDevice(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoOscOutputDevice to quickly initialize a large list.
 */
VuoList_VuoOscOutputDevice VuoListCreateWithCount_VuoOscOutputDevice(const unsigned long count, const VuoOscOutputDevice value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoOscOutputDevice VuoListCopy_VuoOscOutputDevice(const VuoList_VuoOscOutputDevice list);

/**
 * Returns the @ref VuoOscOutputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoOscOutputDevice VuoListGetValue_VuoOscOutputDevice(const VuoList_VuoOscOutputDevice list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoOscOutputDevice *VuoListGetData_VuoOscOutputDevice(const VuoList_VuoOscOutputDevice list);

/**
 * Changes the @ref VuoOscOutputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoOscOutputDevice(const VuoList_VuoOscOutputDevice list, const VuoOscOutputDevice value, const unsigned long index);

/**
 * Inserts the `VuoOscOutputDevice` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoOscOutputDevice(const VuoList_VuoOscOutputDevice list, const VuoOscOutputDevice value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoOscOutputDevice(VuoList_VuoOscOutputDevice list, const VuoOscOutputDevice value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoOscOutputDevice(VuoList_VuoOscOutputDevice list, const VuoOscOutputDevice value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoOscOutputDevice(VuoList_VuoOscOutputDevice list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoOscOutputDevice_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoOscOutputDevice(VuoList_VuoOscOutputDevice list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoOscOutputDevice_areEqual(const VuoList_VuoOscOutputDevice a, const VuoList_VuoOscOutputDevice b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoOscOutputDevice_isLessThan(const VuoList_VuoOscOutputDevice a, const VuoList_VuoOscOutputDevice b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoOscOutputDevice(VuoList_VuoOscOutputDevice list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoOscOutputDevice(VuoList_VuoOscOutputDevice list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoOscOutputDevice(VuoList_VuoOscOutputDevice list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoOscOutputDevice(VuoList_VuoOscOutputDevice list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoOscOutputDevice(VuoList_VuoOscOutputDevice list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoOscOutputDevice(VuoList_VuoOscOutputDevice list);

/**
 * Removes the `VuoOscOutputDevice` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoOscOutputDevice(VuoList_VuoOscOutputDevice list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoOscOutputDevice(const VuoList_VuoOscOutputDevice list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoOscOutputDevice VuoList_VuoOscOutputDevice_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoOscOutputDevice_getJson(const VuoList_VuoOscOutputDevice value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoOscOutputDevice_getSummary(const VuoList_VuoOscOutputDevice value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoOscOutputDevice VuoList_VuoOscOutputDevice_makeFromString(const char *str);
char * VuoList_VuoOscOutputDevice_getString(const VuoList_VuoOscOutputDevice value);
/// @}

/**
 * @}
 */

#endif
