/**
 * @file
 * VuoList_VuoTimeUnit C type definition.
 *
 * @copyright Copyright © 2012–2016 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoTimeUnit_H
#define VuoList_VuoTimeUnit_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoTimeUnit VuoList_VuoTimeUnit
 * A list of @ref VuoTimeUnit elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoTimeUnit_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoTimeUnit elements.
 */
typedef const struct VuoList_VuoTimeUnit_struct { void *l; } * VuoList_VuoTimeUnit;
/// @}
#endif

/**
 * Creates a new list of @ref VuoTimeUnit elements.
 */
VuoList_VuoTimeUnit VuoListCreate_VuoTimeUnit(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoTimeUnit to quickly initialize a large list.
 */
VuoList_VuoTimeUnit VuoListCreateWithCount_VuoTimeUnit(const unsigned long count, const VuoTimeUnit value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoTimeUnit VuoListCopy_VuoTimeUnit(const VuoList_VuoTimeUnit list);

/**
 * Returns the @ref VuoTimeUnit at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoTimeUnit VuoListGetValue_VuoTimeUnit(const VuoList_VuoTimeUnit list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoTimeUnit *VuoListGetData_VuoTimeUnit(const VuoList_VuoTimeUnit list);

/**
 * Changes the @ref VuoTimeUnit at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoTimeUnit(const VuoList_VuoTimeUnit list, const VuoTimeUnit value, const unsigned long index);

/**
 * Inserts the `VuoTimeUnit` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoTimeUnit(const VuoList_VuoTimeUnit list, const VuoTimeUnit value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoTimeUnit(VuoList_VuoTimeUnit list, const VuoTimeUnit value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoTimeUnit(VuoList_VuoTimeUnit list, const VuoTimeUnit value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoTimeUnit(VuoList_VuoTimeUnit list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoTimeUnit_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoTimeUnit(VuoList_VuoTimeUnit list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoTimeUnit_areEqual(const VuoList_VuoTimeUnit a, const VuoList_VuoTimeUnit b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoTimeUnit_isLessThan(const VuoList_VuoTimeUnit a, const VuoList_VuoTimeUnit b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoTimeUnit(VuoList_VuoTimeUnit list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoTimeUnit(VuoList_VuoTimeUnit list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoTimeUnit(VuoList_VuoTimeUnit list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoTimeUnit(VuoList_VuoTimeUnit list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoTimeUnit(VuoList_VuoTimeUnit list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoTimeUnit(VuoList_VuoTimeUnit list);

/**
 * Removes the `VuoTimeUnit` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoTimeUnit(VuoList_VuoTimeUnit list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoTimeUnit(const VuoList_VuoTimeUnit list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoTimeUnit VuoList_VuoTimeUnit_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoTimeUnit_getJson(const VuoList_VuoTimeUnit value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoTimeUnit_getSummary(const VuoList_VuoTimeUnit value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoTimeUnit VuoList_VuoTimeUnit_makeFromString(const char *str);
char * VuoList_VuoTimeUnit_getString(const VuoList_VuoTimeUnit value);
/// @}

/**
 * @}
 */

#endif
