/**
 * @file
 * VuoList_VuoVideoFrame C type definition.
 *
 * @copyright Copyright © 2012–2016 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoVideoFrame_H
#define VuoList_VuoVideoFrame_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoVideoFrame VuoList_VuoVideoFrame
 * A list of @ref VuoVideoFrame elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoVideoFrame_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoVideoFrame elements.
 */
typedef const struct VuoList_VuoVideoFrame_struct { void *l; } * VuoList_VuoVideoFrame;
/// @}
#endif

/**
 * Creates a new list of @ref VuoVideoFrame elements.
 */
VuoList_VuoVideoFrame VuoListCreate_VuoVideoFrame(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoVideoFrame to quickly initialize a large list.
 */
VuoList_VuoVideoFrame VuoListCreateWithCount_VuoVideoFrame(const unsigned long count, const VuoVideoFrame value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoVideoFrame VuoListCopy_VuoVideoFrame(const VuoList_VuoVideoFrame list);

/**
 * Returns the @ref VuoVideoFrame at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoVideoFrame VuoListGetValue_VuoVideoFrame(const VuoList_VuoVideoFrame list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoVideoFrame *VuoListGetData_VuoVideoFrame(const VuoList_VuoVideoFrame list);

/**
 * Changes the @ref VuoVideoFrame at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoVideoFrame(const VuoList_VuoVideoFrame list, const VuoVideoFrame value, const unsigned long index);

/**
 * Inserts the `VuoVideoFrame` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoVideoFrame(const VuoList_VuoVideoFrame list, const VuoVideoFrame value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoVideoFrame(VuoList_VuoVideoFrame list, const VuoVideoFrame value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoVideoFrame(VuoList_VuoVideoFrame list, const VuoVideoFrame value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoVideoFrame(VuoList_VuoVideoFrame list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoVideoFrame_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoVideoFrame(VuoList_VuoVideoFrame list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoVideoFrame_areEqual(const VuoList_VuoVideoFrame a, const VuoList_VuoVideoFrame b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoVideoFrame_isLessThan(const VuoList_VuoVideoFrame a, const VuoList_VuoVideoFrame b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoVideoFrame(VuoList_VuoVideoFrame list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoVideoFrame(VuoList_VuoVideoFrame list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoVideoFrame(VuoList_VuoVideoFrame list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoVideoFrame(VuoList_VuoVideoFrame list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoVideoFrame(VuoList_VuoVideoFrame list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoVideoFrame(VuoList_VuoVideoFrame list);

/**
 * Removes the `VuoVideoFrame` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoVideoFrame(VuoList_VuoVideoFrame list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoVideoFrame(const VuoList_VuoVideoFrame list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoVideoFrame VuoList_VuoVideoFrame_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoVideoFrame_getJson(const VuoList_VuoVideoFrame value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoVideoFrame_getSummary(const VuoList_VuoVideoFrame value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoVideoFrame VuoList_VuoVideoFrame_makeFromString(const char *str);
char * VuoList_VuoVideoFrame_getString(const VuoList_VuoVideoFrame value);
/// @}

/**
 * @}
 */

#endif
